module integrals
  implicit none

contains

  function trap_int(a,b,f,tol) ! recursive approximation of integral
    real :: trap_int ! by trapezoidal rule
    real, intent(in) :: a, b, tol ! integration interval and tolerance

    interface
      function f(x) ! function to integrate
      implicit none
      real :: f
      real, intent(in) :: x
      end function f
    end interface

    integer, parameter :: maxsteps = 2**23
    integer :: steps, i
    real :: acc, dx, prev_estimate, estimate

    steps = 2
    prev_estimate = 0.0 ; estimate = huge(0.0)
    dx = (b - a)*0.5
    acc = (f(a) + f(b))*0.5

    conv: do while (abs(estimate - prev_estimate) > tol)
      prev_estimate = estimate
      do i=1, steps, 2 ! only contributions from new points
        acc = acc + f(a + i*dx)
      end do
      estimate = acc*dx
      steps = steps*2
      if (steps > maxsteps) exit conv
      dx = dx*0.5
    end do conv

    trap_int = estimate

  end function trap_int

end module
